######################################################################
# function performs delay extraction
#    inputs:
#	fname:	name of a ngspice-format output file
#	vc:	voltage threshold for crossing
#    returns:
#	vals:	delays for each waveform in the file
#
proc delays {fname vc {scale 1e12}} {
    set f [open $fname r]
    set d [read $f]
    close $f
# read the file
    set n 0
    foreach line [split $d "\n"] {
	if {[string length $line] == 0} continue
	if {[scan [lindex $line 0] %d N] > 0} {
	    set n [llength $line]
	    set time($N) [expr [lindex $line 1]*$scale]
	    for {set i 1} {$i < $n} {} {
		set data($N,$i) [lindex $line [incr i]]
	    }
	}
    }
    if {$n > 0} {
	set nvar [expr $n-2]
    } else {
	error "file $fname has no data"
    }
# calculate the crossings
    set vals {}
    for {set idx 1} {$idx <= $nvar} {incr idx} {
	set temp {}
	set t -1
        for {set j 0; set i 1} {$i <= $N} {incr i; incr j} {
            set v1 [expr $data($i,$idx)-$vc]
            set v2 [expr $data($j,$idx)-$vc]
            if {[expr $v1*$v2] <= 0.0} {
                set t [expr $time($j)-$v2*($time($i)-$time($j))/($v1-$v2)]
	    }
	}
	lappend vals [format "%.4f" $t]
    }
    return $vals
}
