#!/usr/bin/tclsh
if {$argc < 6} {
    puts stderr {expected to get 6 arguments
	VDD (volts)
	Length (nm)
	name of BSIM model file
	sigma of delta L (nm)
	sigma of VT (volts)
	sigma of mobility (percentage)}
    exit
}
set VDD [lindex $argv 0]
set LEN [lindex $argv 1]
set MOD [lindex $argv 2]
set DLS [lindex $argv 3]
set DLV [lindex $argv 4]
set DLU [lindex $argv 5]
set DEBUG 0
######################################################################
source model.tcl
source delay.tcl
source pton.tcl
source simulate.tcl
######################################################################
# this file is used to generate models
set TEMP [format "temp.%d" [pid]]
######################################################################
# first read the model file
read_model_file $MOD
# stash the nominal values of the parameters
set NOM(xl) $nmos(xl)
set NOM(vn) $nmos(vth0)
set NOM(un) $nmos(u0)
set NOM(vp) $pmos(vth0)
set NOM(up) $pmos(u0)
# stash the sigmas of the parameters
set SIG(xl) $DLS
set SIG(vn) $DLV
set SIG(un) [expr $nmos(u0)*$DLU]
set SIG(vp) $DLV
set SIG(up) [expr $pmos(u0)*$DLU]
######################################################################
# first step is to compute the nominal delay
set nominal [simulate $VDD $LEN $MOD]
puts "nominal delay is $nominal"
######################################################################
# perform a sensitivity analysis, perturb each parameter one at a time
# and see what direction the delay moves in
foreach param {xl vn un vp up} {
    set old $NOM($param)
    set NOM($param) [expr $old + abs($old)*0.2]
    puts " adjusted $param from $old to $NOM($param)"
    write_model_file $TEMP $NOM(xl) $NOM(vn) $NOM(un) $NOM(vp) $NOM(up)
    set delay [simulate $VDD $LEN $TEMP]
    puts "  resulting delay is $delay (delta = [expr $delay-$nominal]"
    if {$delay > $nominal} {
	set SENSE($param) 1
    } else {
	set SENSE($param) -1
    }
    puts "   resulting sensitivity SENSE($param) = $SENSE($param)"
    set NOM($param) $old
}
######################################################################
# now we want to find the smallest value of sigma for which the buffer
# fails. this is indicated by the delay being negative
set XLO 0.0
set XHI 30.0
while {($XHI-$XLO) > 0.1} {
    set XMID [expr 0.5*($XLO+$XHI)]
    foreach param {xl vn un vp up} {
	set NEW($param) [expr $NOM($param)+$XMID*$SIG($param)*$SENSE($param)]
    }
    puts " at $XMID sigma: $NEW(xl) $NEW(vn) $NEW(un) $NEW(vp) $NEW(up)"
    write_model_file $TEMP $NEW(xl) $NEW(vn) $NEW(un) $NEW(vp) $NEW(up)
    set delay [simulate $VDD $LEN $TEMP]
    puts "  resulting delay is $delay"
    if {$delay < 0} {
	set XHI $XMID
    } else {
	set XLO $XMID
    }
    puts "   new search interval $XLO $XHI"
}

file delete $TEMP x.spice x.wave
puts ""
puts [format "failure sigma = %g" [expr 0.5*($XLO+$XHI)]]
exit
