######################################################################
# simulate a chain of inverters, return the delay
#   VDD	    power supply in voltes
#   LEN	    device length (nm)
#   MODEL   name of BSIM model file
#
proc simulate {VDD LEN MODEL } {
    global DEBUG
# compute device dimensions
    set WN [expr $LEN*8]
    set WP [expr $LEN*8*[pton $LEN]]
# create spice file, a sequence of 7 inverters fed by a single pulse, and
# loaded with a 20fF capacitor. print the waveforms at the innermost two
# inverters, one will be rising, and the other falling
    set f [open x.spice w]
    puts $f "* spice file
.include $MODEL
vdd  vdd  0  $VDD
vin  v0   0  0.0 pwl(0n 0,  10p 0,  60p $VDD,  3n $VDD)
.subckt  inv in out vdd
m1  out  in  vdd  vdd  pmos  l=${LEN}nm  w=${WP}nm
m2  out  in  0    0    nmos  l=${LEN}nm  w=${WN}nm
.ends inv
x1  v0  v1  vdd  inv
x2  v1  v2  vdd  inv
x3  v1  v3  vdd  inv
x4  v1  v4  vdd  inv
x5  v1  v5  vdd  inv
c1  v2  0  20fF
c2  v3  0  20fF
c3  v4  0  20fF
c4  v5  0  20fF
.ic v(v1)=$VDD v(v2)=0 v(v3)=0 v(v4)=0 v(v5)=0
.opti nopage
.width out=240
.tran 0.1n 50n
.print tran v(v1)
.end"
    close $f
# run NGSPICE
    file delete x.wave
    catch {exec ngspice -b x.spice > x.wave} msg
    if {$DEBUG} {
	puts "ngspice result: $msg"
    }
# return the delay
    return [delays x.wave [expr 0.5*$VDD]]
}
